#include <stdio.h>
#include <stdlib.h>
#include <sys/time.h>
#include <time.h>

#include "vvl_defs.h"
#include "cpia.h"
#include "alpha_num.h"
#include "mcam.h"
#include "txtfont.h"
#include "s14font.h"

extern int debug_flag;
extern char *frame_dir;
extern int use_utc_flag;
extern int clock_position;
extern int selected_font;

/*
text_to_gstream adds a text at pos x, y in the picture at buffer in gstream format
format in memory.
Copyright Jan Mourer
*/
int text_to_gstream(\
	unsigned char *text, int x, int y,\
	int fr, int fg, int fb,\
	int br, int bg, int bb,\
	BYTE __huge *buffer)
{
int a, c, i, j;
int xsize, ysize;
int size;
int line_length;
unsigned char *font;
int fheight, fwidth;
register bits, row, column, yp, xp;
BYTE __huge *rowptr;
BYTE __huge *plotptr;
unsigned char *fp;
double fy, fu, fv;
double by, bu, bv;
double cr, cg, cb, cu, cv;
int u_time;

if(debug_flag)
	{
	fprintf(stderr,\
	"text_to_gstream(): arg text=%s\n\
	x=%d y=%d\n\
	fr=%d fg=%d fb=%d\n\
	br=%d bg=%d bb=%d\n\
	buffer=%lu\n",\
	text, x, y, fr, fg, fb, br, bg, bb, buffer);
	}

/* argument check */
if(! text) return 0;
if(! buffer) return 0;
if(x < 0) return 0;
if(y < 0) return 0;

/* get x and y size from frame buffer header */
if (buffer[VIDEO_SIZE] == VIDEOSIZE_CIF)
	{
	xsize = (buffer[COL_END] - buffer[COL_START]) << 3;
	ysize = (buffer[ROW_END] - buffer[ROW_START]) << 2;
	}
else
	{
	xsize = (buffer[COL_END] - buffer[COL_START]) << 2;
	ysize = (buffer[ROW_END] - buffer[ROW_START]) << 1;
	}
if(debug_flag)
	{
	fprintf(stderr, "text_to_gstream(): xsize=%d ysize=%d\n",\
	xsize, ysize);
	}
	
size = UNKNOWN;
if( (xsize == 352) && (ysize == 288) ) size = BIG;
if( (xsize == 176) && (ysize == 144) ) size = SMALL;
if(size == UNKNOWN)
	{
	fprintf(stderr, "text_to_gstream(): xsize=%d ysize=%d aborting\n",\
	xsize, ysize);
	return 0;
	}

/* specify a font */
switch(selected_font)
	{
	case TXT_FONT:
		font = txtfont;
		fwidth = TXT_CHAR_WIDTH;
		fheight = TXT_CHAR_HEIGHT;
		break;
	case S14_FONT:
		font = s14font;
		fwidth = S14_CHAR_WIDTH;
		fheight = S14_CHAR_HEIGHT;
		break;
	}/* end switch selected_font */

/* 
format is YUYV starting at gStreamBuffer + 64
format line: length length YUYV.... EOL
gstream line length is 2 + 352 * 2 + EOL or 2 + 176 * 2 + EOL
*/
line_length = 2 + (xsize * 2) + 1; 

/* color standard */
/* Y spec */
cr = 0.3;
cg = 0.59;
cb = 0.11;

/* U spec */
cu = .5 / (1.0 - cb);

/* V spec */
cv = .5 / (1.0 - cr);

/* fore ground y */
fy = cr * fr + cg * fg + cb * fb;
fy = (219.0 / 256.0) * fy + 16.5;  /* nominal range: 16..235 */

/* back ground y */
by = cr * br + cg * bg + cb * bb;
by = (219.0 / 256.0) * by + 16.5;  /* nominal range: 16..235 */

/* fore ground  u */
fu = cu * (fb - fy);
fu = (224.0 / 256.0) * fu + 128.5; /* 16..240 */
/* fu = -0.16874 * fr - 0.33126 * fg + 0.50000 * fb + 128;*/
  
/* back ground  u */
bu = cu * (bb - by);
bu = (224.0 / 256.0) * bu + 128.5; /* 16..240 */
/* bu = -0.16874 * br - 0.33126 * bg + 0.50000 * bb + 128;*/
  
/* fore ground v */
fv = cv * (fr - fy);
fv = (224.0 / 256.0) * fv + 128.5; /* 16..240 */
/* fv = 0.50000 * fr - 0.41869 * fg - 0.08131 * fb + 128;*/
			
/* back ground v */
bv = cv * (br - by);
bv = (224.0 / 256.0) * bv + 128.5; /* 16..240 */
/* bv = 0.50000 * br - 0.41869 * bg - 0.08131 * bb + 128;*/
			
while (*text)
	{
	fp = font + fheight * *(unsigned char *)text;
	for (row = 0; row < fheight; row++)
		{
		yp = y + row;

		/* y boundary check */
		if(yp >= (ysize - 1) ) continue;

		/* calculate start top left character */
		rowptr = buffer + 64 + 2 + (yp * line_length);

		bits = *fp++;
		for (column = 0; column < fwidth; column++)
			{
			xp = x + column;
			
			/* x boundary check */
			if(xp >= (xsize - 1) ) continue;

			/* calculate the address in the buffer */
			plotptr = rowptr + (xp * 2);			

			/* for even x positions U, and for odd x positions V */
			if( (xp % 2) == 0) u_time = 1;
			else u_time = 0;

			if(bits & (0x80 >> column))
				{
				/* plot a dot in foreground color */				
				*plotptr = fy;

				/* print u or v */
				if(u_time) *(plotptr + 1) = fu;
				else *(plotptr + 1) = fv;
				}
			else		
				{
				/* plot a dot in background color */
				*plotptr = by;
				
				/* print u or v */
				if(u_time) *(plotptr + 1) = bu;
				else *(plotptr + 1) = bv;
				}
			}/* end for all columns in character */
		}/* end for all rows in character */
	text++;
	x += fwidth;
	}/* end while all characters in text */

return 1;
}/* end function text_to_gstream */ 


char *get_time_as_string(int short_format_flag, int ut_flag)
{
time_t now;
static char temp[1024];
struct tm *universal_time, *local_time;
char *ptr;

if(debug_flag)
	{
	fprintf(stderr,\
	"get_time_as_string(): arg short_format_flag=%d ut_flag=%d\n",\
	short_format_flag, ut_flag);
	}

/* 
decided to adapt to universal time (UT == GMT).
New format:
from rfc1036.html:
Wdy, DD Mon YY HH:MM:SS TIMEZONE 
*/

now = time(0);
if(ut_flag) 
	{
	universal_time = gmtime(&now);
	if(short_format_flag)
		{
		strftime (temp, 511, "%b %d %y %H:%M:%S UT", universal_time);
		}
	else
		{
		strftime (temp, 511, "%a, %d %b %Y %H:%M:%S UTC", universal_time);
		}
	}
else /* local time */
	{
	local_time = localtime(&now);
	if(short_format_flag)
		{
		strftime (temp, 511, "%b %d %Y %H:%M:%S", local_time);
		}
	 else
	 	{
		strftime (temp, 511, "%c", local_time);
		}
	}
ptr = strsave(temp);
if(! ptr)
	{
	fprintf(stderr, "get_time_as_string(): cannot allocate space for tempn");
	return 0;
	}

if(debug_flag)
	{
	fprintf(stderr, "get_time_as_string(): ptr=%s\n", ptr);
	}

return(ptr);
}/* end function get_time_as_string */


/*
add_clock adds a clock with time and date in the correct place and format in
a gstream format buffer.
*/
int add_clock(BYTE __huge *buffer)
{
int xsize, ysize;
int size;
BYTE __huge *ptr;
int text_xpos, text_ypos;

if(debug_flag)
	{
	fprintf(stderr, "add_clock(): arg buffer=%lu\n", buffer);
	}

/* get x and y size from frame buffer header */
if(buffer[VIDEO_SIZE] == VIDEOSIZE_CIF)
	{
	xsize = (buffer[COL_END] - buffer[COL_START]) << 3;
	ysize = (buffer[ROW_END] - buffer[ROW_START]) << 2;
	}
else
	{
	xsize = (buffer[COL_END] - buffer[COL_START]) << 2;
	ysize = (buffer[ROW_END] - buffer[ROW_START]) << 1;
	}
if(debug_flag)
	{
	fprintf(stderr, "add_clock(): xsize=%d ysize=%d\n",\
	xsize, ysize);
	}
	
size = UNKNOWN;
if( (xsize == 352) && (ysize == 288) ) size = BIG;
if( (xsize == 176) && (ysize == 144) ) size = SMALL;
if(size == UNKNOWN)
	{
	fprintf(stderr, "add_clock(): xsize=%d ysize=%d aborting\n",\
	xsize, ysize);
	return 0;
	}

/* get the time */
if(size == BIG)
	{
	ptr = (char *)get_time_as_string(0, use_utc_flag);
	}
else
	{
	ptr = (char *)get_time_as_string(1, use_utc_flag);
	}

if(! ptr) return 0;

/* print the time at the required position */

/* short_format_flag, ut_flag */
/* print short or long format time */
if(clock_position == TOP_LEFT)
	{
	text_xpos = 0;
	text_ypos = 0;
	}
else if(clock_position == TOP_MIDDLE)
	{
	text_xpos = (xsize - (strlen(ptr) * TXT_CHAR_WIDTH) ) / 2;
	text_ypos = 0;
	}
else if(clock_position == TOP_RIGHT)
	{
	text_xpos = xsize - (strlen(ptr) * TXT_CHAR_WIDTH) - 1;
	text_ypos = 0;
	}
else if(clock_position == BOTTOM_LEFT)
	{
	text_xpos = 0;
	text_ypos = ysize - TXT_CHAR_HEIGHT; 
	}
else if(clock_position == BOTTOM_MIDDLE)
	{
	text_xpos = (xsize - (strlen(ptr) * TXT_CHAR_WIDTH) ) / 2;
	text_ypos = ysize - TXT_CHAR_HEIGHT; 
	}
else if(clock_position == BOTTOM_RIGHT)
	{
	text_xpos = xsize - (strlen(ptr) * TXT_CHAR_WIDTH) - 1;
	text_ypos = ysize - TXT_CHAR_HEIGHT; 
	}

/* time to gstream buffer */
if(! text_to_gstream(ptr, text_xpos, text_ypos, 255, 255, 255, 0, 0, 0, buffer) )
	{
	free(ptr);
	return 0;
	}

/* free the time string, was allocated by get_time_as_string */
free(ptr);

return 1;
}/* end fuction add_clock */


